==> ltspfs_0.5.3.orig.tar.gz: ltspfs-0.5.3/src/cdpinger.c <==
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/cdrom.h>
#include <glib.h>

#define POLL_INTERVAL 3				/* Check cdrom every 3 secs */

/*
 * enum for our mount states
 */

typedef enum {
    CDP_CDROM_MOUNT,
    CDP_CDROM_UMOUNT
} CdpMountType;

/*
 * check_cd_status:  Switch mounted and unmounted states in the cdrom, and
 * call out our shell scripts to handle the actual work of plumbing the
 * ltspfs connection.
 */

gboolean
check_cd_status(gpointer device)
{
    char dev[BUFSIZ], cmd[BUFSIZ];
    int cdrom, status;
    static CdpMountType last = CDP_CDROM_UMOUNT;

    snprintf(dev, sizeof dev, "/dev/%s", (char *)device);

    /*
     * Try to open the cdrom device.  It might not exist, for a usb cdrom,
     * so simply return false so we exit.  Device has been unplugged.
     * Make sure it's unmounted, and exit.
     */

    if ((cdrom = open(dev, O_RDONLY|O_NONBLOCK)) < 0) {
        snprintf(cmd, sizeof cmd,
                "/lib/udev/ltspfs_entry remove %s", (char *)device);
        g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
        return FALSE;
    }

    /*
     * Talk to the cdrom, get the status, and close it.
     */

    ioctl(cdrom, CDROM_LOCKDOOR, 0);
    status = ioctl (cdrom, CDROM_DRIVE_STATUS, 0);
    close(cdrom);

    /*
     * Something badly wrong with the cdrom.  Don't bother talking to it
     * anymore.
     */

    if (status < 0) {
        snprintf(cmd, sizeof cmd,
                "/lib/udev/ltspfs_entry remove %s", (char *)device);
        return g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
        return FALSE;
    }

    /*
     * Look at cdrom, see if we've got a disk or not, and mount or umount
     * appropriately
     */

    switch(status) {
        case CDS_NO_DISC:
        case CDS_TRAY_OPEN:
            if (last == CDP_CDROM_MOUNT) {
                snprintf(cmd, sizeof cmd,
                        "/lib/udev/ltspfs_entry remove %s", (char *)device);
                last = CDP_CDROM_UMOUNT;
                return g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
            }
            break;
        case CDS_DISC_OK:
            if (last == CDP_CDROM_UMOUNT) {
                snprintf(cmd, sizeof cmd,
                        "/lib/udev/ltspfs_entry add %s auto", (char *)device);
                last = CDP_CDROM_MOUNT;
                return g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
            }
            break;
    }

    return TRUE;
}

/*
 * end_cd_watch: Callback routine error routine for the timeout.  If
 * check_cd_status has returned false, somthing's happened, like, say
 * the USB cdrom being unplugged, so just exit the main loop.
 */

void
end_cd_watch(gpointer loop)
{
    g_main_loop_quit((GMainLoop *)loop);
}

int
main(int argc, char *argv[])
{
    GMainLoop *loop;

    if (argc != 2)
        exit(1);

    daemon(0,0);

    loop = g_main_loop_new (NULL, FALSE);

    /* Add a timeout */
    g_timeout_add_full(G_PRIORITY_DEFAULT, POLL_INTERVAL * 1000,
            (GSourceFunc)check_cd_status, argv[1],
	    (GDestroyNotify)end_cd_watch);

    g_main_loop_run(loop);

    return 0;
}

==> ltspfs_0.5.5.orig.tar.gz: ltspfs-0.5.5/src/cdpinger.c <==
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/cdrom.h>
#include <glib.h>

#define POLL_INTERVAL 3				/* Check cdrom every 3 secs */

/*
 * enum for our mount states
 */

typedef enum {
    CDP_CDROM_MOUNT,
    CDP_CDROM_UMOUNT
} CdpMountType;

char *device;

/*
 * check_cd_status:  Switch mounted and unmounted states in the cdrom, and
 * call out our shell scripts to handle the actual work of plumbing the
 * ltspfs connection.
 */

gboolean
check_cd_status()
{
    char dev[BUFSIZ], cmd[BUFSIZ];
    int cdrom, status;
    static CdpMountType last = CDP_CDROM_UMOUNT;

    snprintf(dev, sizeof dev, "/dev/%s", device);

    /*
     * Try to open the cdrom device.  It might not exist, for a usb cdrom,
     * so simply return false so we exit.  Device has been unplugged.
     * Make sure it's unmounted, and exit.
     */

    if ((cdrom = open(dev, O_RDONLY|O_NONBLOCK)) < 0) {
        if (last == CDP_CDROM_MOUNT) {
            snprintf(cmd, sizeof cmd,
                    "/lib/udev/ltspfs_entry remove %s", device);
            g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
        }
        return FALSE;
    }

    /*
     * Talk to the cdrom, get the status, and close it.
     */

    ioctl(cdrom, CDROM_LOCKDOOR, 0);
    status = ioctl (cdrom, CDROM_DRIVE_STATUS, 0);
    close(cdrom);

    /*
     * Something badly wrong with the cdrom.  Don't bother talking to it
     * anymore.
     */

    if (status < 0) {
        snprintf(cmd, sizeof cmd,
                "/lib/udev/ltspfs_entry remove %s", device);
        return g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
        return FALSE;
    }

    /*
     * Look at cdrom, see if we've got a disk or not, and mount or umount
     * appropriately
     */

    switch(status) {
        case CDS_NO_DISC:
        case CDS_TRAY_OPEN:
            if (last == CDP_CDROM_MOUNT) {
                snprintf(cmd, sizeof cmd,
                        "/lib/udev/ltspfs_entry remove %s", device);
                last = CDP_CDROM_UMOUNT;
                return g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
            }
            break;
        case CDS_DISC_OK:
            if (last == CDP_CDROM_UMOUNT) {
                snprintf(cmd, sizeof cmd,
                        "/lib/udev/ltspfs_entry add %s auto", device);
                last = CDP_CDROM_MOUNT;
                return g_spawn_command_line_sync(cmd, NULL, NULL, NULL, NULL);
            }
            break;
    }

    return TRUE;
}

/*
 * end_cd_watch: Callback routine error routine for the timeout.  If
 * check_cd_status has returned false, somthing's happened, like, say
 * the USB cdrom being unplugged, so just exit the main loop.
 */

void
end_cd_watch(gpointer loop)
{
    g_main_loop_quit((GMainLoop *)loop);
}

int
main(int argc, char *argv[])
{
    GMainLoop *loop;

    if (argc != 2)
        exit(1);

    g_type_init();

    device = argv[1];

    daemon(0,0);

    loop = g_main_loop_new (NULL, FALSE);

    /* Add a timeout */
    g_timeout_add_full(G_PRIORITY_DEFAULT, POLL_INTERVAL * 1000,
            (GSourceFunc)check_cd_status, NULL,
	    (GDestroyNotify)end_cd_watch);

    g_main_loop_run(loop);

    return 0;
}
